@@ -1,3 +1,5 @@
+package org.toontown.teapop;
+
 /**
  * User data for use with teapop server
  * This interface is provided for reference because teapop does not check the interface,
@@ -6,6 +8,11 @@
 
 public interface User {
 
+  /**
+   * Get path to user mailbox
+   *
+   * @return path to mailbox relative to domain directory
+   */
   public String getMailbox();
 
-}
\ No newline at end of file
+}
