@@ -1,7 +1,7 @@
-/* $Id: //depot/Teapop/0.3/teapop/pop_cmd_capa.c#4 $ */
+/* $Id: //depot/Teapop/0.3/teapop/pop_cmd_capa.c#5 $ */
 
 /*
- * Copyright (c) 1999-2001 ToonTown Consulting
+ * Copyright (c) 1999-2002 ToonTown Consulting
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -27,6 +27,8 @@
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
+#include <sys/types.h>
+
 #include <stdio.h>
 
 #include "config.h"
@@ -51,8 +53,19 @@ pop_cmd_capa(arg, pinfo)
 	pop_socket_send(pinfo->out, "USER");
 	pop_socket_send(pinfo->out, "LOGIN-DELAY 900");
 	pop_socket_send(pinfo->out, "UIDL");
-	pop_socket_send(pinfo->out, "EXPIRE %s",
-	    (pinfo->autodelete == 1) ? "0" : "NEVER");
-	pop_socket_send(pinfo->out, "IMPLEMENTATION Teapop-%s", POP_VERSION);
+	/*
+	 * The EXPIRE response must always be the lowest value we can
+	 * guarantee a message is retained. This means that if we are
+	 * running with -d/-D the response will be 0, even if there is
+	 * a -w specified.
+	 */
+	if (pinfo->autodelete != 0)
+		pop_socket_send(pinfo->out, "EXPIRE 0");
+	else if (pinfo->expire != 0)
+		pop_socket_send(pinfo->out, "EXPIRE %d", pinfo->expire);
+	else
+		pop_socket_send(pinfo->out, "EXPIRE NEVER");
+	pop_socket_send(pinfo->out, "IMPLEMENTATION Teapop%s-%s",
+	     ((pinfo->ssl != 0) ? "SSL" : ""), POP_VERSION);
 	pop_socket_send(pinfo->out, ".");
 }
