@@ -1,7 +1,7 @@
-/* $Id: //depot/Teapop/0.3/teapop/pop_lock.c#9 $ */
+/* $Id: //depot/Teapop/0.3/teapop/pop_lock.c#10 $ */
 
 /*
- * Copyright (c) 1999-2001 ToonTown Consulting
+ * Copyright (c) 1999-2003 ToonTown Consulting
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -46,6 +46,9 @@
 #include <unistd.h>
 
 #include "teapop.h"
+#ifdef NEED_FLOCK
+#include "flock.h"
+#endif
 #include "pop_lock.h"
 
 POP_INFO *xpinfo;
@@ -58,6 +61,8 @@ pop_lock_maildrop(pinfo, loop)
 	int retval = 0;
 	int locks = 0;
 
+	struct stat sb;
+
 	/* This is for reference in pop_unlock_maildrop() */
 	xpinfo = pinfo;
 
@@ -128,7 +133,7 @@ pop_lock_maildrop(pinfo, loop)
 			    "(errno = %d)", pinfo->maildrop,
 			    (pinfo->chroot[0] == '\0' ? "NO_CHROOT" :
 			    pinfo->chroot), errno);
-			return 2;
+			return (2);
 		}
 
 		retval = pop_lock_dotlock(pinfo);
@@ -142,9 +147,9 @@ pop_lock_maildrop(pinfo, loop)
 				syslog(LOG_NOTICE, "Problems opening mailbox "
 				    "%s in %s (errno = %d)", pinfo->maildrop,
 				    pinfo->chroot, errno);
-				return 2;
+				return (2);
 			}
-			return 0;
+			return (0);
 		}
 		
 		if ((locks & LOCK_FLOCK) && (retval == 0))
@@ -153,6 +158,12 @@ pop_lock_maildrop(pinfo, loop)
 			retval = pop_lock_fcntl(pinfo);
 		if ((locks & LOCK_LOCKF) && (retval == 0))
 			retval = pop_lock_lockf(pinfo);
+
+		pinfo->mboxperm = fstat(fileno(pinfo->mbox), &sb) < 0 ?
+		    0 : sb.st_mode;
+		pinfo->mboxperm &= (S_ISUID | S_ISGID | S_ISVTX | S_IRWXU |
+		    S_IRWXG | S_IRWXO);
+
 	}
 
 	/*
@@ -170,6 +181,7 @@ pop_lock_dotlock(pinfo)
 	POP_INFO *pinfo;
 {
 	int fd;
+	char hostname[128];
 
 	struct stat sb;
 	time_t current;
@@ -185,13 +197,12 @@ pop_lock_dotlock(pinfo)
 			/*
 			 * If chroot already has the ending slash this can
 			 * show 2 slashes in syslog but its only visual problem
-			 */  
-			syslog(LOG_ERR, "Can't create lockfile %s in %s%s%s "
+			 */
+			syslog(LOG_ERR, "Can't create lockfile %s in %s/%s "
 			    "(errno = %d)", pinfo->dotlock, 
-			    (pinfo->chroot[0] == '\0' ? "" : pinfo->chroot),
-			    (pinfo->chroot[0] == '\0' ? "" : "/"),
-			    pinfo->maildrop, errno);
-			return 2;
+			    pinfo->chroot, pinfo->maildrop,
+			    errno);
+			return (2);
 		}
 
 		/*
@@ -202,40 +213,65 @@ pop_lock_dotlock(pinfo)
 			/* Checking for, and remove, stale lock */
 			if ((stat(pinfo->dotlock, &sb)) != -1) {
 				current = time(NULL);
-				if ((sb.st_mtime +
-				    (pinfo->locktimeout)) < current) {
-					if (unlink(pinfo->dotlock) == -1)
-						return 1;
-					else
-						return 1;
+				if ((sb.st_mtime + pinfo->locktimeout) 
+				    < current) {
+					if (unlink(pinfo->dotlock) == -1) {
+						syslog(LOG_ERR, 
+						   "Error removing "
+						   "lockfile %s in %s/%s "
+						   " (errno = %d)",
+						   pinfo->dotlock,
+						   pinfo->chroot,
+						   pinfo->maildrop,
+						   errno);
+						return (1);
+					} else {
+						if (pinfo->domain[0] != '\0')
+							syslog(LOG_ERR,
+							   "Lockfile removed for %s@%s",
+							   pinfo->userid,
+							   pinfo->domain);
+						else
+							syslog(LOG_ERR,
+							   "Lockfile removed for %s",
+							   pinfo->userid);
+						/*
+						 * lockfile can be created now
+						 */ 
+						return pop_lock_dotlock(pinfo);
+					}
 				}
+			} else {
+				/* We can't check the lock file!? */
+				syslog(LOG_ERR, "Failed doing stat() on "
+				    "lockfile %s in %s/%s (errno = %d)",
+				    pinfo->dotlock, 
+				    pinfo->chroot, pinfo->maildrop, 
+				    errno);
+				return (2);
 			}
-		} else {
-			/*
-			 * If the error creating the lockfile is not ENOENT,
-			 * then something fishy is going on, and a syslog()
-			 * message should be written to alert the admin.
-			 */
-			if (errno != ENOENT) {
-				syslog(LOG_NOTICE, "Problems with lockfile %s "
-				    "in %s (errno = %d)", pinfo->maildrop,
-				    pinfo->chroot, errno);
- 				return 2;
-			}
-			return 1;
 		}
+		syslog(LOG_ERR, "Lockfile %s already exists in %s/%s",
+		    pinfo->dotlock, pinfo->chroot, pinfo->maildrop);
+		return (1);
 	}
 
 	/* Associate the stream pinfo->lock with the file descriptor. */
 	if ((pinfo->lock = fdopen(fd, "w")) == NULL) {
-		syslog(LOG_NOTICE, "Problems with lockfile %s in %s "
-		    "(errno = %d)", pinfo->maildrop, pinfo->chroot, errno);
-		return 2;
+		syslog(LOG_NOTICE, "Problems with lockfile %s/%s in %s "
+		    "(errno = %d)", pinfo->maildrop, pinfo->dotlock,
+		    pinfo->chroot, errno);
+		return (2);
 	}
 
+	(void)gethostname((char *)&hostname, sizeof(hostname));
+	(void)fprintf(pinfo->lock, "%d %s %s %s\n", (int)getpid(), hostname, 
+	     pinfo->remotehost, pinfo->remoteip);
+	(void)fflush(pinfo->lock);
+	
 	xpinfo->locktrack |= LOCK_DOTLOCK;
 
-	return 0;
+	return (0);
 }
 
 int
@@ -249,11 +285,11 @@ pop_lock_fcntl(pinfo)
 
 	if ((fcntl(fileno(pinfo->mbox), F_SETLK, &lock_data)) == -1) {
 		(void)fclose(pinfo->mbox);
-		return 1;
+		return (1);
 	}
 
 	xpinfo->locktrack |= LOCK_FCNTL;
-	return 0;
+	return (0);
 }
 
 int
@@ -262,10 +298,10 @@ pop_lock_flock(pinfo)
 {
 	if ((flock(fileno(pinfo->mbox), LOCK_EX | LOCK_NB)) == -1) {
 		(void)fclose(pinfo->mbox);
-		return 1;
+		return (1);
 	}
 	xpinfo->locktrack |= LOCK_FLOCK;
-	return 0;
+	return (0);
 }
 
 int
@@ -275,11 +311,11 @@ pop_lock_lockf(pinfo)
 
 	if ((lockf(fileno(pinfo->mbox), F_TLOCK, 0)) == -1) {
 		(void)fclose(pinfo->mbox);
-		return 1;
+		return (1);
 	}
 
 	xpinfo->locktrack |= LOCK_LOCKF;
-	return 0;
+	return (0);
 }
 
 void
@@ -294,6 +330,12 @@ pop_unlock_maildrop(void)
 	/* Block all signals to avoid this function being run twice */
 	sigprocmask(SIG_BLOCK, (sigset_t *)xpinfo->smask, NULL);
 
+	/* Reset permissions on mbox (which might get lost when writing) */
+	if (xpinfo->mboxtype == 0) {
+		fflush(xpinfo->mbox);
+		fchmod(fileno(xpinfo->mbox), xpinfo->mboxperm);
+	}
+ 
 	/* Only unlock each lock if it's locked */
 
 	if (xpinfo->locktrack & LOCK_DOTLOCK) {
