@@ -1,7 +1,7 @@
-/* $Id: //depot/Teapop/0.3/teapop/pop_maildir.c#7 $ */
+/* $Id: //depot/Teapop/0.3/teapop/pop_maildir.c#8 $ */
 
 /*
- * Copyright (c) 2000-2001 ToonTown Consulting
+ * Copyright (c) 2000-2003 ToonTown Consulting
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -26,6 +26,12 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
+ 
+/*
+ * Information about Maildir can be found at :
+ *     http://cr.yp.to/proto/maildir.html
+ *     http://mirrors.dataloss.nl/www.qmail.org/man/man5/maildir.html
+ */
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -36,6 +42,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <syslog.h>
+#include <unistd.h>
 
 #include "config.h"
 
@@ -49,12 +56,9 @@
 #include "pop_maildir.h"
 
 /*
- * TODO: Like other maildir programs the name of file can be used to define the 
- * UIDL of message, because message filename is unique
- * Here is a problem because the routine that calculate UIDL also count the 
- * number of lines that is need in pop_dnld.c
- * the need for caching UIDL is increasing because users does not like the 
- * delay when connecting to the pop server
+ * TODO: support for info flags in message filename
+ * Check with people if old UIDL calculation can be removed to leave
+ * only FASTMAILDIR implementation which is working very fine here
  */
 
 /*
@@ -68,7 +72,8 @@ typedef struct _maildir_info
 	char buf[1024];
 	char realname[256];
 /*
- * Used to name the message files with unique name
+ * Used to name the message files with unique name when necessary
+ * not used directly here, but in patches for this code
  */
 	char hostname[128];
 	int pid;
@@ -88,7 +93,7 @@ POP_INFO *pinfo;
 char *dir;
 char *fname;
 {
-	int counter, finduidl, retval;
+	int counter, finduidl, retval, i;
 	char *ptr;
 	unsigned char digest[16];
 	FILE *fd;
@@ -106,7 +111,7 @@ char *fname;
 
 	retval = lstat(minfo->realname, &sb);
 	if (retval != 0 || !S_ISREG(sb.st_mode))
-		return 0;
+		return (0);
 
 	/*
 	 * XXX - Add a sanity check so the filename
@@ -117,51 +122,79 @@ char *fname;
 	 * Maildir specification say to ignore files beginning with '.'
 	 */
 	if (fname[0] == '.')
-		return 0;
+		return (0);
 
 	if ((curmsg = malloc(sizeof(POP_MSG))) == NULL) {
 		syslog(LOG_ERR,
 		    "Error allocating memory in add_maildir_message : %d",
 		    errno);
-		return 1;
+		return (1);
 	}
-	bzero(curmsg, sizeof(POP_MSG));
+	memset(curmsg, 0, sizeof(POP_MSG));
 	if ((ptr = malloc((strlen(minfo->realname) + 1) *
 		    sizeof(char))) == NULL) {
 		free(curmsg);
 		syslog(LOG_ERR,
 		    "Error allocating memory in add_maildir_message : %d",
 		    errno);
-		return 1;
+		return (1);
 	}
-	(void) strcpy(ptr, minfo->realname);
+	(void)strcpy(ptr, minfo->realname);
 	curmsg->file = ptr;
 
-	if ((fd = fopen(curmsg->file, "r")) == NULL) {
-		syslog(LOG_ERR, "Error opening message file : %s %d",
-		    curmsg->file, errno);
-		free(ptr);
-		free(curmsg);
-		return 0;
-	}
+	/* 
+	 * Get the timestamp from the filename 
+	 * standard defines filename as time.pid.host
+	 * if it does not work use value from lstat
+	 */
+	curmsg->created = (time_t)strtol(fname, (char **)NULL, 10);
+	if (curmsg->created == 0)
+		curmsg->created = sb.st_mtime;
+
 	finduidl = pinfo->useuidl;
 	MD5Init(&ctx);
 /*
  * When FAST_MAILDIR is defined it will calculate the UIDL from filename
  * instead of using the data on file
+ * Using a number of lines greater than real message size will work
  */	
 #ifdef FAST_MAILDIR
-	MD5Update(&ctx, (unsigned char *) curmsg->file, strlen(curmsg->file));
-	(void) fseek(fd, 0, SEEK_END);
-	curmsg->size = ftell(fd);
-	curmsg->lines = 99999999; 
+        ptr = strchr(fname, ':');
+        if (ptr == NULL)
+		MD5Update(&ctx, fname, strlen(fname));
+	else
+		MD5Update(&ctx, fname, (ptr - fname) );
+	curmsg->size = sb.st_size;
+	curmsg->lines = 999999999;
 #else
+	if ((fd = fopen(curmsg->file, "r")) == NULL) {
+		syslog(LOG_ERR, "Error opening message file : %s %d",
+		    curmsg->file, errno);
+		free(ptr);
+		free(curmsg);
+		return (0);
+	}
 	while (fgets(minfo->buf, sizeof(minfo->buf), fd)) {
-		if (minfo->buf[strlen(minfo->buf) - 1] == '\n') {
+		/* Find all NULL characters and replace with questionmarks */
+		i = strlen(minfo->buf);
+		while ((i == 0 || minfo->buf[i-1] != '\n') && i < (sizeof(minfo->buf) - 1)) {
+			minfo->buf[i] = '?';
+			i = strlen(minfo->buf);
+		}
+ 
+		/*
+		 * If line only ends with LF, we need to add one to the size
+		 * since we will be sending CRLF to the client.
+		 * Note we need to check that the number of characters in
+		 * buf is high enough (found in i) to not cause buffer
+		 * underflow.
+		 */
+		if ((i > 0 && minfo->buf[i-1] == '\n') &&
+		    (i < 1 || minfo->buf[i-2] != '\r')) {
 			curmsg->size++;
 			curmsg->lines++;
-		} else if (strlen(minfo->buf) != sizeof(minfo->buf))
-			minfo->buf[strlen(minfo->buf)] = '?';
+		}
+
 		if (finduidl == 1 && !strncasecmp(minfo->buf, "X-UIDL: ", 8)) {
 			minfo->buf[strlen(minfo->buf) - 1] = '\0';
 			strncpy(curmsg->uidl, &minfo->buf[8],
@@ -173,8 +206,8 @@ char *fname;
 		MD5Update(&ctx, (unsigned char *) minfo->buf,
 		    strlen(minfo->buf));
 	}
-#endif
 	(void) fclose(fd);
+#endif
 
 	MD5Final(digest, &ctx);
 	if (!(finduidl == 0 && pinfo->useuidl == 1)) {
@@ -189,20 +222,113 @@ char *fname;
 		pinfo->firstmsg = minfo->lastmsg = curmsg;
 	else
 		minfo->lastmsg = minfo->lastmsg->nextmsg = curmsg;
+	minfo->msgcount += 1;
 
-	return 0;
+	return (0);
 }
 
+int 
+pop_maildir_load_dir(minfo, pinfo, dir)
+MAILDIR_INFO *minfo;
+POP_INFO *pinfo;
+char *dir;
+{
+	char procdir[BIGSTRING];
+	struct dirent *dp;
+	DIR *dirp;
+
+	dirp = opendir(dir);
+	(void) strncpy(procdir, dir, sizeof(procdir));
+	(void) strncat(procdir, "/", sizeof(procdir)-strlen(procdir));
+
+	if (dirp == NULL) {
+		syslog(LOG_ERR, "can't open maildir %s (%d)", 
+		    procdir, errno);
+		return (1);
+	}
+
+	while ((dp = readdir(dirp)) != NULL) {
+
+		if (add_maildir_message(minfo, pinfo, procdir,
+			dp->d_name) == 1) {
+			break;
+		}
+	}
+	(void) closedir(dirp);
+	
+	return (0);
+}
+
+#ifdef SORT_MAILDIR
+typedef struct _sort_data
+{
+	POP_MSG *msg;
+}
+SORT_DATA;
+
+
+/*
+ * Make comparision for qsort
+ */
+int 
+compare_msg(sd1, sd2)
+SORT_DATA *sd1;
+SORT_DATA *sd2;
+{
+    if (sd1->msg->created < sd2->msg->created) 
+    	return (-1);
+    else if (sd1->msg->created == sd2->msg->created)
+    	return (0);
+    else
+    	return (1);
+    	
+}
+
+/*
+ * Sort message list by creating date
+ * create array with the pointers and use qsort to sort messages
+ */
+void 
+pop_maildir_sort(pinfo, cnt)
+POP_INFO *pinfo;
+int cnt;
+{
+	POP_MSG *msg;
+	
+	if (cnt > 1) {
+		int x = 0;
+		SORT_DATA *data;
+
+		data = malloc(sizeof(SORT_DATA)*cnt);
+		msg = pinfo->firstmsg;
+		while (msg) {
+			data[x].msg = msg;
+			x += 1;
+			msg = msg->nextmsg;
+		}
+
+		qsort(data, cnt, sizeof(SORT_DATA), compare_msg);
+		
+		x = 0;
+		pinfo->firstmsg = data[x].msg;
+		while (x < (cnt-1)) {
+			data[x].msg->nextmsg = data[x+1].msg;
+			data[x+1].msg->nextmsg = NULL;
+			x += 1;
+		}
+
+		free(data);
+	}
+}
+
+#endif /* SORT_MAILDIR */
+
 int
 pop_maildir_get_status(pinfo)
 POP_INFO *pinfo;
 {
-	int i;
-	char procdir[10];
 
-	DIR *dirp;
 	MAILDIR_INFO minfo;
-	struct dirent *dp;
 
 	/*
 	 * Initialize the MAILDIR_INFO
@@ -219,34 +345,18 @@ POP_INFO *pinfo;
 		minfo.lastmsg = NULL;
 	else {
 		minfo.lastmsg = pinfo->firstmsg;
-		while (minfo.lastmsg->nextmsg != NULL)
+		while (minfo.lastmsg->nextmsg != NULL) {
 			minfo.lastmsg = minfo.lastmsg->nextmsg;
+			minfo.msgcount += 1;
+		}
 	}
 
-	for (i = 0; i < 2; i++) {
-		if (i == 0) {
-			dirp = opendir("cur");
-			(void) strncpy(procdir, "cur/", sizeof(procdir));
-		} else if (i == 1) {
-			dirp = opendir("new");
-			(void) strncpy(procdir, "new/", sizeof(procdir));
-		} else
-			return 1;
+	(void) pop_maildir_load_dir(&minfo, pinfo, "cur");
+	(void) pop_maildir_load_dir(&minfo, pinfo, "new");
 
-		if (dirp == NULL) {
-			syslog(LOG_ERR, "can't open maildir/%d %s (%d)", i,
-			    procdir, errno);
-			continue;
-		}
+#ifdef SORT_MAILDIR	
+	pop_maildir_sort(pinfo, minfo.msgcount);
+#endif	
 
-		while ((dp = readdir(dirp)) != NULL) {
-
-			if (add_maildir_message(&minfo, pinfo, procdir,
-				dp->d_name) == 1) {
-				break;
-			}
-		}
-		(void) closedir(dirp);
-	}
-	return 0;
+	return (0);
 }
