@@ -1,7 +1,7 @@
-/* $Id: //depot/Teapop/0.3/teapop/pop_mysql.c#4 $ */
+/* $Id: //depot/Teapop/0.3/teapop/pop_mysql.c#5 $ */
 
 /*
- * Copyright (c) 2000-2001 ToonTown Consulting
+ * Copyright (c) 2000-2003 ToonTown Consulting
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -29,6 +29,9 @@
 
 #include "config.h"
 
+#include <sys/types.h>
+
+#include <ctype.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <syslog.h>
@@ -50,7 +53,7 @@
 
 #ifndef SOCKETFILE
 #define SOCKETFILE NULL
-#endif 
+#endif
 
 extern int he_is_mysql_user;
 
@@ -79,7 +82,7 @@ POP_AUTH_SQL	*psql;
 {
 	int i ;
 
-	i = 0; 
+	i = 0;
 	while (i < TRY_N) {	/* Try it 10 Times  */
 		mysql_init(&real_mysql);
 		if (!(mysql=mysql_real_connect(&real_mysql, psql->host,
@@ -95,18 +98,18 @@ POP_AUTH_SQL	*psql;
 	}
 
 	if (mysql != NULL) {
-		i = 0; 
+		i = 0;
 		while (i < TRY_N) {	/* Try it 10 Times */
 			if  (mysql_select_db(mysql, psql->db)) {
 			} else {
-				return(0);
+				return (0);
 			}
 			i++;
 			sleep(1);
 		}
 	}
 
-	return(-1);
+	return (-1);
 }
 
 
@@ -114,7 +117,7 @@ POP_AUTH_SQL	*psql;
 void
 My_Mysql_Close(int keepopen_flag)
 {
-	if (keepopen_flag == 0) { 
+	if (keepopen_flag == 0) {
 		/* Close the connection after each record */
 		if (mysql != NULL) {
 			mysql_close(mysql);
@@ -133,8 +136,8 @@ pop_password_mysql(pinfo, auth, isapop, passwd)
 {
 	POP_AUTH_SQL	*psql;
 	MD5_CTX		ctx;
-	char	*ptr, buf2[512], *ppass, md5pass[32];
-	int	nTuples, counter;
+	char	*ptr, buf2[512], *ppass, md5pass[32], euserid[BIGSTRING*2+1];
+	int	counter;
 	unsigned char digest[16];
 	MYSQL_RES *result;
 	MYSQL_ROW row;
@@ -150,18 +153,20 @@ pop_password_mysql(pinfo, auth, isapop, passwd)
 		if (err==-1) {
 			syslog(LOG_ERR,
 			    "Cannot connect to MySQL Server (ret -1)");
-			return 1;
+			return (1);
 		}
 	}
 	if (!mysql) {
 		syslog(LOG_ERR, "Cannot connect to MySQL Server (ret 0)");
-		return 1;
+		return (1);
 	}
 
 	/* ===================== */
-	sprintf(mybuf, "SELECT %s, %s%s%s FROM %s WHERE %s = '%s'", 
+	mysql_real_escape_string(mysql, euserid, pinfo->userid,
+	    strlen(pinfo->userid));
+	sprintf(mybuf, "SELECT %s, %s%s%s FROM %s WHERE %s = '%s'",
 	    psql->userrow, psql->passrow, (*(psql->mailrow) == NULL ? " " : ", "),
-	    psql->mailrow, psql->table, psql->userrow, pinfo->userid);
+	    psql->mailrow, psql->table, psql->userrow, euserid);
 
 	len = strlen(psql->userrow);
 	if (check_host) {
@@ -183,20 +188,26 @@ pop_password_mysql(pinfo, auth, isapop, passwd)
 		strcat(mybuf, " AND active=1 AND start_date<=curdate() AND expire_date>curdate()" );
 	}
 
-	/* --- Query! --- */ 
+	/* --- Query! --- */
 	if (mysql_query(mysql, mybuf) < 0) {
+		My_Mysql_Close(mysql_keepopen);
 		syslog(LOG_ERR, "Error in Query record (check fields names)");
-		return 1;
+		return (1);
 	}
 	if (!(result=mysql_store_result(mysql))) {
+		My_Mysql_Close(mysql_keepopen);
 		syslog(LOG_ERR, "Query returned nothing - check MySQL server");
-		return 1;
+		return (1);
 	}
 	if (mysql_num_rows(result) < 1) {
+		My_Mysql_Close(mysql_keepopen);
 		syslog(LOG_ERR, "Query result is incomplete");
-		return 1;
+		return (1);
 	}
 
+	/* Successful query, close the connection nicely */
+	My_Mysql_Close (mysql_keepopen);
+
 	/* OK. Parse result */
 	row = mysql_fetch_row(result);
 	len = strlen(row[0]) + 1;
@@ -212,7 +223,7 @@ pop_password_mysql(pinfo, auth, isapop, passwd)
 	if (isapop == 1) { /* Was 1 */
 		snprintf(buf2, sizeof(buf2), "%s%s", pinfo->apopstr, ppass);
 		MD5Init(&ctx);
-		MD5Update(&ctx, (unsigned char *)buf2, strlen(buf2)); 
+		MD5Update(&ctx, (unsigned char *)buf2, strlen(buf2));
 		MD5Final(digest, &ctx);
 		ptr = md5pass;
 		for (counter = 0; counter < sizeof(digest); counter++, ptr += 2)
@@ -222,8 +233,8 @@ pop_password_mysql(pinfo, auth, isapop, passwd)
 
 	encpw = crypt((char *)passwd, (char *)ppass);
 
-	if ((isapop == 0 && !strcmp(ppass, passwd)) ||
-	   (isapop == 1 && !strcmp(ppass, md5pass)) ||
+	if ((isapop == 0 && !strcmp(passwd, ppass)) ||
+	   (isapop == 1 && !strcmp(passwd, md5pass)) ||
 	   !(strcmp (encpw, ppass))) {
 		strncpy(pinfo->chroot, auth->maildrop, sizeof(pinfo->chroot));
 		if (psql->mailrow[0] != '\0') {
@@ -238,7 +249,7 @@ pop_password_mysql(pinfo, auth, isapop, passwd)
 			strncpy(pinfo->maildrop, pinfo->userid,
 			    sizeof(pinfo->maildrop));
 		}
-		return 0;
+		return (0);
 	}
-	return 1;
+	return (1);
 }
