@@ -1,7 +1,7 @@
-/* $Id: //depot/Teapop/0.3/teapop/pop_passwd.c#4 $ */
+/* $Id: //depot/Teapop/0.3/teapop/pop_passwd.c#5 $ */
 
 /*
- * Copyright (c) 1999-2001 ToonTown Consulting
+ * Copyright (c) 1999-2003 ToonTown Consulting
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -46,6 +46,9 @@
 #include <syslog.h>
 #include <unistd.h>
 
+#ifdef HAVE_CRYPT_H
+#include <crypt.h>
+#endif
 
 #ifdef HAVE_SHADOW_H
 #include <shadow.h>
@@ -79,7 +82,11 @@ enum authtypes {
 	REJECT
 };
 
-int pop_password_passwd __P((char *, char *, char *, int, char *));
+int pop_droppriv __P((POP_INFO *, POP_AUTH *));
+int pop_password_ldap __P((POP_INFO *, POP_AUTH *, int, char *));
+int pop_password_mysql __P((POP_INFO *, POP_AUTH *, int, char *));
+int pop_password_passwd __P((POP_INFO *, POP_AUTH *, int, char *));
+int pop_password_pgsql __P((POP_INFO *, POP_AUTH *, int, char *));
 int pop_password_textfile __P((POP_INFO *, POP_AUTH *, int, char *, int));
 
 int
@@ -93,22 +100,26 @@ pop_read_pwdinfo(pinfo)
 	char *phostname, *pport, *pdatabase;	/* Used for SQLs */
 	char *pdbuser, *pdbpass, *ptable;	/*     ditto     */
 	char *puserrow, *ppassrow, *pmailrow;	/*     ditto     */
+#ifdef HAVE_LDAP
 	char *prootdn, *pbind;			/* Used for LDAPAUTH */
+#endif
 
 	FILE *fd;
 	POP_AUTH *tmpauth, *lastauth;
 	POP_AUTH_TEXT *ptextfile, *phtpasswd;
 	POP_AUTH_SQL *psql;
+#ifdef HAVE_LDAP
 	POP_AUTH_LDAP *pldap;
+#endif
 
 	struct hostent *hp;
 	lastauth = NULL;	/* Make -Wall happy */
 
 	if ((tmpauth = malloc(sizeof(POP_AUTH))) == NULL) {
 		syslog(LOG_CRIT, "memory problems");
-		return 1;
+		return (1);
 	}
-	bzero(tmpauth, sizeof(POP_AUTH));
+	memset(tmpauth, 0, sizeof(POP_AUTH));
 
 	if ((fd = fopen(PWDINFO, "r")) == NULL) {
 		/*
@@ -132,7 +143,7 @@ pop_read_pwdinfo(pinfo)
 		strcpy(tmpauth->gid, "+");
 
 		pinfo->firstauth = tmpauth;
-		return 0;
+		return (0);
 	}
 
 	line = 0;
@@ -221,9 +232,9 @@ pop_read_pwdinfo(pinfo)
 			lastauth = tmpauth;
 			if ((tmpauth = malloc(sizeof(POP_AUTH))) == NULL) {
 				syslog(LOG_CRIT, "memory problems");
-				return 1;
+				return (1);
 			}
-			bzero(tmpauth, sizeof(POP_AUTH));
+			memset(tmpauth, 0, sizeof(POP_AUTH));
 			continue;
 		}
 
@@ -336,11 +347,11 @@ pop_read_pwdinfo(pinfo)
 
 			ptextfile = malloc(sizeof(POP_AUTH_TEXT));
 			if (ptextfile == NULL) {
-				bzero(tmpauth, sizeof(POP_AUTH));
+				memset(tmpauth, 0, sizeof(POP_AUTH));
 				syslog(LOG_CRIT, "memory problems");
-				return 1;
+				return (1);
 			}
-			bzero(ptextfile, sizeof(POP_AUTH_TEXT));
+			memset(ptextfile, 0, sizeof(POP_AUTH_TEXT));
 			strcpy(ptextfile->file, pfile);
 			ptextfile->max = atoi(pmax);
 			tmpauth->extra = ptextfile;
@@ -391,11 +402,11 @@ pop_read_pwdinfo(pinfo)
 
 			phtpasswd = malloc(sizeof(POP_AUTH_TEXT));
 			if (phtpasswd == NULL) {
-				bzero(tmpauth, sizeof(POP_AUTH));
+				memset(tmpauth, 0, sizeof(POP_AUTH));
 				syslog(LOG_CRIT, "memory problems");
-				return 1;
+				return (1);
 			}
-			bzero(phtpasswd, sizeof(POP_AUTH_TEXT));
+			memset(phtpasswd, 0, sizeof(POP_AUTH_TEXT));
 			strcpy(phtpasswd->file, pfile);
 			phtpasswd->max = atoi(pmax);
 			tmpauth->extra = phtpasswd;
@@ -521,11 +532,11 @@ pop_read_pwdinfo(pinfo)
 
 			psql = malloc(sizeof(POP_AUTH_SQL));
 			if (psql == NULL) {
-				bzero(tmpauth, sizeof(POP_AUTH));
+				memset(tmpauth, 0, sizeof(POP_AUTH));
 				syslog(LOG_CRIT, "memory problems");
-				return 1;
+				return (1);
 			}
-			bzero(psql, sizeof(POP_AUTH_SQL));
+			memset(psql, 0, sizeof(POP_AUTH_SQL));
 			strcpy(tmpauth->uid, puid);
 			strcpy(tmpauth->gid, pgid);
 			if (!strcasecmp(pauth, "mysql"))
@@ -628,11 +639,11 @@ pop_read_pwdinfo(pinfo)
 			/* Create POP_AUTH_LDAP object */
 			pldap = (POP_AUTH_LDAP *)malloc(sizeof(POP_AUTH_LDAP));
 			if (pldap == NULL) {
-				bzero(tmpauth, sizeof(POP_AUTH));
+				memset(tmpauth, 0, sizeof(POP_AUTH));
 				syslog(LOG_CRIT, "memory problem");
-				return 1;
+				return (1);
 			}
-			bzero(pldap, sizeof(POP_AUTH_LDAP));
+			memset(pldap, 0, sizeof(POP_AUTH_LDAP));
 
 			/* Fill data from config file */
 			strncpy(pldap->host, phostname, sizeof(pldap->host)-1);
@@ -672,9 +683,9 @@ pop_read_pwdinfo(pinfo)
 		lastauth = tmpauth;
 		if ((tmpauth = malloc(sizeof(POP_AUTH))) == NULL) {
 			syslog(LOG_CRIT, "memory problems");
-			return 1;
+			return (1);
 		}
-		bzero(tmpauth, sizeof(POP_AUTH));
+		memset(tmpauth, 0, sizeof(POP_AUTH));
 	}
 
 	/* There will always be one unused but malloc()'ed tmpauth left */
@@ -683,7 +694,7 @@ pop_read_pwdinfo(pinfo)
 	/* Close PWDINFO file (rude not to) */
 	fclose(fd);
 
-	return 0;
+	return (0);
 }
 
 int
@@ -692,7 +703,7 @@ pop_verify_password(pinfo, passwd, isapop)
 	char *passwd;
 	int isapop;
 {
-	int state, retval;
+	int state;
 
 	POP_AUTH *curauth;
 	
@@ -731,12 +742,9 @@ pop_verify_password(pinfo, passwd, isapop)
 		    !strcasecmp(curauth->localip,pinfo->localip))) {
 			switch (curauth->authmethod) {
 			case SYSPASSWD:
-				if (isapop == 0)
-					if (pop_password_passwd(pinfo->userid,
-					    passwd, curauth->maildrop,
-					    curauth->hash, pinfo->maildrop)
-					    == 0)
-						state = 3;
+				if (pop_password_passwd(pinfo, curauth,
+				    isapop, passwd) == 0)
+					state = 3;
 				break;
 			case TEXTFILE:
 				if (pop_password_textfile(pinfo, curauth,
@@ -779,7 +787,7 @@ pop_verify_password(pinfo, passwd, isapop)
 #ifdef HAVE_LDAP
 			case LDAPAUTH:
 				if (pop_password_ldap(pinfo, curauth, isapop,
-				    passwd == 0))
+				    passwd) == 0)
 					state = 3;
 				break;
 #endif /* HAVE_LDAP */
@@ -789,7 +797,7 @@ pop_verify_password(pinfo, passwd, isapop)
 			}
 #ifndef KEEPTRYING
 			if (state != 3)
-				return 1;
+				return (1);
 #endif /* KEEPTRYING */
 #ifndef TRYDEFAULT
 			if (state == 2)
@@ -811,35 +819,18 @@ pop_verify_password(pinfo, passwd, isapop)
 
 		/* First run POP-before-SMTP routines and then drop privs */
 		pop_pop_before_smtp(pinfo);
-		return pop_droppriv(pinfo, curauth);
+		return (pop_droppriv(pinfo, curauth));
 	}
 
-	return 1;
-}
-
-char *
-hashed(string, hashlevel)
-	char *string;
-	int hashlevel;
-{
-	int i;
-	static char hash[1024] = "";
-
-	if (hashlevel == 0)
-		return "";
-
-	for (i = 0; i < hashlevel && i < 1000 && string[i] != '\0'; i++) {
-		strncat(hash, &string[i], 1);
-		strcat(hash, "/");
-	}
-
-	return hash;
+	return (1);
 }
 
 int
-pop_password_passwd(userid, passwd, pmaildrop, hash, maildrop)
-	char *userid, *passwd, *pmaildrop, *maildrop;
-	int hash;
+pop_password_passwd(pinfo, auth, isapop, passwd)
+	POP_INFO *pinfo;
+	POP_AUTH *auth;
+	int isapop;
+	char *passwd;
 {
 	char *encpw;
 
@@ -848,45 +839,70 @@ pop_password_passwd(userid, passwd, pmaildrop, hash, maildrop)
 	struct spwd *suserinfo;
 #endif
 
-	if ((userinfo = getpwnam(userid)) == NULL)
-		return 1;
+	/* We can't do APOP with system password, bail out. */
+	if (isapop == 1)
+		return (1);
+
+	if ((userinfo = getpwnam(pinfo->userid)) == NULL)
+		return (1);
 #ifdef HAVE_SHADOW_H
-	if ((suserinfo = getspnam(userid)) == NULL)
-		return 1;
+	if ((suserinfo = getspnam(pinfo->userid)) == NULL)
+		return (1);
 	encpw = crypt(passwd, suserinfo->sp_pwdp);
 	if (strcmp(encpw, suserinfo->sp_pwdp))
-		return 1;
+		return (1);
 #else
 	encpw = crypt(passwd, userinfo->pw_passwd);
 	if (strcmp(encpw, userinfo->pw_passwd))
-		return 1;
+		return (1);
 #endif
 
-	if (pmaildrop == NULL) {
+	if (auth->maildrop == NULL) {
 		/* We are running w/o a teapop.passwd file */
 #ifdef HOMEDIRSPOOL
-		snprintf(maildrop, BIGSTRING, "%s/%s", userinfo->pw_dir,
-		    MAILBOX);
+		strncpy(pinfo->chroot, userinfo->pw_dir,
+		    sizeof(pinfo->chroot) - 1);
+		pinfo->chroot[sizeof(pinfo->chroot) - 1] = '\0';
+		strncpy(pinfo->maildrop, MAILBOX, sizeof(pinfo->maildrop) - 1);
+		pinfo->maildrop[sizeof(pinfo->maildrop) - 1] = '\0';
 #else
-		snprintf(maildrop, BIGSTRING, "%s/%s", MAILSPOOL, userid);
+		strncpy(pinfo->chroot, MAILSPOOL, sizeof(pinfo->chroot) - 1);
+		pinfo->chroot[sizeof(pinfo->chroot) - 1] = '\0';
+		strncpy(pinfo->maildrop, pinfo->userid,
+		    sizeof(pinfo->maildrop) - 1);
+		pinfo->maildrop[sizeof(pinfo->maildrop) - 1] = '\0';
 #endif /* HOMEDIRSPOOL */
 	} else {
-		if (pmaildrop[0] == '~') {
-			snprintf(maildrop, BIGSTRING, "%s%s", userinfo->pw_dir,
-			&pmaildrop[1]);
+		if (auth->maildrop[0] == '~') {
+			strncpy(pinfo->chroot, userinfo->pw_dir,
+			    sizeof(pinfo->chroot) - 1);
+			pinfo->chroot[sizeof(pinfo->chroot) - 1] = '\0';
+			strncpy(pinfo->maildrop, &auth->maildrop[1],
+			    sizeof(pinfo->maildrop) - 1);
+			pinfo->maildrop[sizeof(pinfo->maildrop) - 1] = '\0';
 		} else {
-				if (pmaildrop[strlen(pmaildrop)-1] == '/')
-					snprintf(maildrop, BIGSTRING, "%s%s%s/",
-					    pmaildrop, hashed(userid, hash),
-					    userid);
-				else
-					snprintf(maildrop, BIGSTRING, "%s/%s%s",
-					    pmaildrop, hashed(userid, hash),
-					    userid);
+			strncpy(pinfo->chroot, auth->maildrop,
+			    sizeof(pinfo->chroot) - 1);
+			pinfo->chroot[sizeof(pinfo->chroot) - 1] = '\0';
+			strncpy(pinfo->maildrop, pinfo->userid,
+			    sizeof(pinfo->maildrop) - 1);
+			pinfo->maildrop[sizeof(pinfo->maildrop) - 1] = '\0';
 		}
 	}
 
-	return 0;
+	/*
+	 * If pinfo->chroot ends with a slash, the admin configured
+	 * teapop.passwd to use Maildir. However, we need to move that
+	 * slash over to pinfo->maildrop, since that's what Teapop looks
+	 * for.
+	 */
+	if (pinfo->chroot[0] != '\0' &&
+	    pinfo->chroot[strlen(pinfo->chroot) - 1] == '/') {
+		pinfo->chroot[strlen(pinfo->chroot) - 1] = '\0';
+		strncat(pinfo->maildrop, "/", sizeof(pinfo->maildrop));
+	}
+
+	return (0);
 }
 
 int
@@ -898,7 +914,7 @@ pop_password_textfile(pinfo, auth, isapop, passwd, htpasswd)
 	int htpasswd;
 {
 	int lines, readlines, counter;
-	char *puser, *ppass, *pmbox, *ptr;
+	char *puser, *ppass, *pmbox = NULL, *ptr;
 	char buf[512], buf2[512], md5pass[32];
 	unsigned char digest[16];
 
@@ -913,14 +929,14 @@ pop_password_textfile(pinfo, auth, isapop, passwd, htpasswd)
 	if ((fd = fopen(ptext->file, "r")) == NULL) {
 		syslog(LOG_ERR, "Password file %s doesn't exist "
 		    "or isn't readable.", ptext->file);
-		return 1;
+		return (1);
 	}
 	fstat(fileno(fd), &sb);
 	if (!S_ISREG(sb.st_mode) || sb.st_nlink != 1) {
 		fclose(fd);
 		syslog(LOG_ERR, "Password file %s not a normal file",
 		    ptext->file);
-		return 1;
+		return (1);
 	}
 
 	lines = readlines = 0;
@@ -933,29 +949,29 @@ pop_password_textfile(pinfo, auth, isapop, passwd, htpasswd)
 		}
 		while (buf[strlen(buf)-1] == '\n')
 			buf[strlen(buf)-1] = '\0';
-		while (buf[0] != '\0' && isspace(buf[0]))
+		while (buf[0] != '\0' && isspace((int)buf[0]))
 			strcpy(&buf[0], &buf[1]);
 		if (buf[0] == '#' || buf[0] == ';')
 			continue;
 		if (++lines > ptext->max) {
 			syslog(LOG_ERR, "Reached max number of users for %s",
 			    auth->domain);
-			return 1;
-			    
+			return (1);
+
 		}
 
 		puser = buf;
 		if ((ppass = strchr(puser, ':')) == NULL) {
 			syslog(LOG_ERR, "line %d corrupt in password file for "
 			    "%s", readlines, auth->domain);
-			return 1;
+			return (1);
 		}
 		*ppass++ = '\0';
 		if (htpasswd == 0) {
 			if ((pmbox = strchr(ppass, ':')) == NULL) {
 				syslog(LOG_ERR, "line %d corrupt in password "
 				    "file for %s", readlines, auth->domain);
-				return 1;
+				return (1);
 			}
 			*pmbox++ = '\0';
 		}
@@ -985,7 +1001,7 @@ pop_password_textfile(pinfo, auth, isapop, passwd, htpasswd)
 					strncpy(pinfo->maildrop, pmbox,
 					    sizeof(pinfo->maildrop));
 					fclose(fd);
-					return 0;
+					return (0);
 				}
 			} else {
 				if ((ptr = strchr(ppass, ':')) != NULL)
@@ -997,12 +1013,12 @@ pop_password_textfile(pinfo, auth, isapop, passwd, htpasswd)
 					strncpy(pinfo->maildrop, puser,
 					    sizeof(pinfo->maildrop));
 					fclose(fd);
-					return 0;
+					return (0);
 				}
 			}
 		}
 	}
 
 	fclose(fd);
-	return 1;
+	return (1);
 }
