@@ -1,7 +1,7 @@
-/* $Id: //depot/Teapop/0.3/teapop/pop_pgsql.c#4 $ */
+/* $Id: //depot/Teapop/0.3/teapop/pop_pgsql.c#5 $ */
 
 /*
- * Copyright (c) 1999-2001 ToonTown Consulting
+ * Copyright (c) 1999-2003 ToonTown Consulting
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -29,8 +29,11 @@
 
 #include "config.h"
 
+#include <sys/types.h>
+
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 #include <syslog.h>
 
 #include <libpq-fe.h>
@@ -50,7 +53,7 @@ pop_password_pgsql(pinfo, auth, isapop, passwd)
         int isapop;
         char *passwd;
 {
-	char	*ptr, buf2[512], *ppass, md5pass[32];
+	char	*ptr, buf2[512], *ppass, md5pass[32], euserid[BIGSTRING*2+1];
 	int	nTuples, counter;
 	unsigned char digest[16];
 
@@ -68,7 +71,7 @@ pop_password_pgsql(pinfo, auth, isapop, passwd)
 	if (PQstatus(conn) == CONNECTION_BAD) {
 		syslog(LOG_ERR, "Could not connect to PostgreSQL server");
 		PQfinish(conn);
-		return 1;
+		return (1);
 	}
 
 	res = PQexec(conn, "BEGIN");
@@ -77,18 +80,19 @@ pop_password_pgsql(pinfo, auth, isapop, passwd)
 		    "Problems communicating with PostgreSQL server (BEGIN)");
 		PQclear(res);
 		PQfinish(conn);
-		return 1;
+		return (1);
 	}
 	PQclear(res);
 	
 	if ((ptr = malloc(256)) == NULL) {
 		syslog(LOG_CRIT, "memory problems");
-		return 1;
+		return (1);
 	}
 
+	PQescapeString(euserid, pinfo->userid, strlen(pinfo->userid));
 	snprintf(ptr, 256, "SELECT %s, %s%s%s FROM %s WHERE %s = '%s'",
 	    psql->userrow, psql->passrow, (*(psql->mailrow) == NULL ? " " : ", "),
-	    psql->mailrow, psql->table, psql->userrow, pinfo->userid);
+	    psql->mailrow, psql->table, psql->userrow, euserid);
 	res = PQexec(conn, ptr);
 
 	free(ptr);
@@ -98,14 +102,14 @@ pop_password_pgsql(pinfo, auth, isapop, passwd)
 		    "Problems communicating with PostgreSQL server (SELECT)");
 		PQclear(res);
 		PQfinish(conn);
-		return 1;
+		return (1);
 	}
 	
 	nTuples = PQntuples(res);
 	if (nTuples == 0) {
 		PQclear(res);
 		PQfinish(conn);
-		return 1;
+		return (1);
 	}
 	ppass = PQgetvalue(res,0,1);
 
@@ -120,8 +124,8 @@ pop_password_pgsql(pinfo, auth, isapop, passwd)
 		*ptr   = '\0';
 	}
 
-	if ((isapop == 0 && !strcmp(ppass, passwd)) ||
-	    (isapop == 1 && !strcmp(ppass, md5pass))) {
+	if ((isapop == 0 && !strcmp(passwd, ppass)) ||
+	    (isapop == 1 && !strcmp(passwd, md5pass))) {
 		strncpy(pinfo->chroot, auth->maildrop, sizeof(pinfo->chroot));
 		if (psql->mailrow[0] != '\0') {
 			ptr = PQgetvalue(res, 0, 2);
@@ -136,10 +140,10 @@ pop_password_pgsql(pinfo, auth, isapop, passwd)
 			    sizeof(pinfo->maildrop));
 		PQclear(res);
 		PQfinish(conn);
-		return 0;
+		return (0);
 	}
 
 	PQclear(res);
 	PQfinish(conn);
-	return 1;
+	return (1);
 }
