@@ -1,7 +1,7 @@
-/* $Id: //depot/Teapop/0.3/teapop/pop_socket.c#6 $ */
+/* $Id: //depot/Teapop/0.3/teapop/pop_socket.c#8 $ */
 
 /*
- * Copyright (c) 1999-2001 ToonTown Consulting
+ * Copyright (c) 1999-2003 ToonTown Consulting
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -56,6 +56,16 @@
 #include "pop_socket.h"
 #include "pop_strings.h"
 
+/** BASED ON sshd.c FROM openssh.com */
+#ifdef HAVE_TCPD_H
+#include <tcpd.h>
+#endif
+#ifdef WITH_TCPD
+int allow_severity = LOG_INFO;
+int deny_severity = LOG_WARNING;
+static char *progname = "teapop";
+#endif
+
 void
 #if __STDC__
 pop_socket_send(FILE * fd, const char *fmt, ...)
@@ -117,20 +127,20 @@ va_dcl
 	if ((ptr = va_arg(ap, char *)) == NULL)
 	{
 		va_end(ap);
-		return -1;
+		return (-1);
 	}
 
 	for (;;) {
 		alarm(timeout);
 		if (setjmp(env)) {
 			va_end(ap);
-			return 0;
+			return (0);
 		}
 		if ((fgets(buf, 255, stdin)) == NULL) {
 			if (feof(stdin))
 				pop_signal_sigpipe(SIGPIPE);
 			else
-				return -1;
+				return (-1);
 		}
 
 		if (buf[strlen(buf) - 1] != '\n') {
@@ -150,7 +160,7 @@ va_dcl
 				strncpy(arguments,
 				    buf + strlen(ptr) + 1, size - 1);
 				arguments[size - 1] = '\0';
-				return arg;
+				return (arg);
 			}
 			ptr = va_arg(ap, char *);
 			arg++;
@@ -165,7 +175,7 @@ va_dcl
 	}
 
 	va_end(ap);
-	return -1;
+	return (-1);
 }
 
 int
@@ -174,32 +184,97 @@ POP_INFO *pinfo;
 {
 	socklen_t len;
 
+#ifdef INET6
+	struct sockaddr_storage ss;
+	char hbuf[NI_MAXHOST];
+#else
 	struct sockaddr_in name;
 	struct hostent *hp;
+#endif
 
+#ifdef INET6
+	len = sizeof(ss);
+#else
 	len = sizeof(name);
+#endif
+
 
 	/* Get our own IP */
+#ifdef INET6
+	if (getsockname(pinfo->insck, (struct sockaddr *)&ss, &len) == -1) {
+		syslog(LOG_ERR, "can't do getsockname() (errno = %d)", errno);
+		return (1);
+	}
+#else
 	if (getsockname(pinfo->insck, (struct sockaddr *) &name, &len) == -1) {
 		syslog(LOG_ERR, "can't do getsockname() (errno = %d)", errno);
-		return 1;
+		return (1);
 	}
-	strncpy(pinfo->localip, (char *) inet_ntoa(name.sin_addr),
+#endif
+
+#ifdef INET6
+	/* XXX - Is there an AF-independent way of doing this?? */
+	switch (ss.ss_family) {
+	case AF_INET:
+		inet_ntop(ss.ss_family,
+		    &((struct sockaddr_in *)&ss)->sin_addr, hbuf, NI_MAXHOST);
+		break;
+	case AF_INET6:
+		inet_ntop(ss.ss_family,
+		    &((struct sockaddr_in6 *)&ss)->sin6_addr, hbuf, NI_MAXHOST);
+		break;
+	default:
+		syslog(LOG_ERR, "Unknown AF type on socket");
+	}
+	strncpy(pinfo->localip, hbuf, sizeof(pinfo->localip));
+	pinfo->localip[sizeof(pinfo->localip)] = '\0';
+#else
+	strncpy(pinfo->localip, (char *)inet_ntoa(name.sin_addr),
 	    sizeof(pinfo->localip));
+#endif
 	pinfo->localip[sizeof(pinfo->localip)] = '\0';
 
 	/* Get IP of the remote client */
+#ifdef INET6
+	if (getpeername(pinfo->insck, (struct sockaddr *)&ss, &len) == -1) {
+		syslog(LOG_ERR, "can't get socket and/or do getpeername "
+		    "(error = %d)", errno);
+		return (1);
+	}
+	if (getnameinfo((struct sockaddr *)&ss, len, hbuf,
+		     sizeof(hbuf), NULL, 0, NI_NUMERICHOST)) {
+			syslog(LOG_NOTICE,
+			    "can't get IP of remote client (error = %d)",
+			    errno);
+	} else {
+		strncpy(pinfo->remoteip, hbuf, sizeof(pinfo->remoteip));
+		pinfo->remoteip[sizeof(pinfo->remoteip)] = '\0';
+	}
+#else
 	if (getpeername(pinfo->insck, (struct sockaddr *) &name, &len) == -1) {
 		syslog(LOG_ERR, "can't get socket and/or do getpeername "
 		    "(error = %d)", errno);
-		return 1;
+		return (1);
 	}
 	strncpy(pinfo->remoteip, (char *) inet_ntoa(name.sin_addr),
 	    sizeof(pinfo->remoteip));
 	pinfo->remoteip[sizeof(pinfo->remoteip)] = '\0';
+#endif
 
 	/* Now resolv the IP and see what we get */
 	if (pinfo->nodns < 2) {
+#ifdef INET6
+		if (getnameinfo((struct sockaddr *)&ss, len, hbuf,
+		     sizeof(hbuf), NULL, 0, NI_NAMEREQD)) {
+			syslog(LOG_NOTICE,
+			    "can't do reverse dns on client (error = %d)",
+			    errno);
+		} else {
+			strncpy(pinfo->remotehost, hbuf,
+			    sizeof(pinfo->remotehost));
+			pinfo->remotehost[sizeof(pinfo->remotehost)] = '\0';
+		}
+#else
 		hp = gethostbyaddr((char *) &name.sin_addr,
 		    sizeof(name.sin_addr), AF_INET);
 		if (hp == NULL) {
@@ -211,6 +286,7 @@ POP_INFO *pinfo;
 			    sizeof(pinfo->remotehost));
 			pinfo->remotehost[sizeof(pinfo->remotehost)] = '\0';
 		}
+#endif
 	}
 	if (pinfo->remotehost[0] == '\0') {
 		strncpy(pinfo->remotehost, "unknown",
@@ -218,8 +294,23 @@ POP_INFO *pinfo;
 		pinfo->remotehost[sizeof(pinfo->remotehost)] = '\0';
 	}
 
+#ifdef WITH_TCPD
+	{
+		struct request_info req;
+
+		request_init(&req, RQ_DAEMON, progname, RQ_FILE, pinfo->insck, NULL);
+		fromhost(&req);
+
+		if (!hosts_access(&req)) {
+			syslog(LOG_ERR,
+			    "tcp_wrappers connection refused %s %s ",
+			    pinfo->remoteip,pinfo->remotehost);
+			return (2);
+		}
+	}
+#endif
 	pinfo->out = fdopen(pinfo->outsck, "wb");
-	return 0;
+	return (0);
 }
 
 int
@@ -233,28 +324,28 @@ pop_socket_bind(void)
 		syslog(LOG_ERR, "Problem calling socket(): %s",
 		    strerror(errno));
 		perror("socket");
-		return 0;
+		return (0);
 	}
 	/* westr - allow startup with reusing the sockets */
 	if ((setsockopt(sckfd, SOL_SOCKET, SO_REUSEADDR, (char *)&sck_opt,
 	    sizeof(sck_opt))) < 0) {
 		perror("setsockopt");
-		return 0;
+		return (0);
 	}
-	bzero((char *) &name, sizeof(struct sockaddr_in));
+	memset((char *) &name, 0, sizeof(struct sockaddr_in));
 	name.sin_family = AF_INET;
-	name.sin_port = htons(110);
+	name.sin_port = htons(POP3PORT);
 	if ((bind(sckfd, (struct sockaddr *) &name, sizeof(name))) < 0) {
 		syslog(LOG_ERR, "Problem calling bind(): %s",
 		    strerror(errno));
 		perror("bind");
-		return 0;
+		return (0);
 	}
 	if ((listen(sckfd, 10)) < 0) {
 		syslog(LOG_ERR, "Problem calling listen(): %s",
 		    strerror(errno));
 		perror("listen");
-		return 0;
+		return (0);
 	}
 	/*
 	 * XXX - for now we do not set the socket nonblock since it
@@ -265,10 +356,10 @@ pop_socket_bind(void)
 		syslog(LOG_ERR, "Problem calling fcntl(): %s",
 		    strerror(errno));
 		perror("fcntl");
-		return 0;
+		return (0);
 	}
 */
-	return sckfd;
+	return (sckfd);
 }
 
 int
@@ -290,5 +381,5 @@ int sckfd;
          Will return number of FDs ready (1) on success.
          Will return 0 on timeout, -1 on error.
        */
-       return select(sckfd+1,&rfds,0,0,NULL);
+       return (select(sckfd+1,&rfds,0,0,NULL));
 }
