@@ -1,7 +1,7 @@
-/* $Id: //depot/Teapop/0.3/teapop/teapop.c#6 $ */
+/* $Id: //depot/Teapop/0.3/teapop/teapop.c#7 $ */
 
 /*
- * Copyright (c) 1999-2001 ToonTown Consulting
+ * Copyright (c) 1999-2003 ToonTown Consulting
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -38,6 +38,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <syslog.h>
+#include <time.h>
 #include <unistd.h>
 
 #include "config.h"
@@ -72,11 +73,12 @@ char *argv[];
 {
 	extern char *optarg;
 	int ch, standalone, retval;
+	char syslogname[20];
 
 	POP_INFO pinfo;
 
 	argv[0] = "teapop";
-	bzero(&pinfo, sizeof(pinfo));
+	memset(&pinfo, 0, sizeof(pinfo));
 	standalone = 0;		/* runs from inetd if there is no -s */
 	pinfo.ignoreimap = 0;	/* Don't ignore UW IMAP's message by default */
 	pinfo.timeout = 900;	/* Disconnect an inactive user after 15 mins */
@@ -88,10 +90,10 @@ char *argv[];
 	signal(SIGUSR1, s_monitor);
 #endif
 
-	openlog("teapop", LOG_PID | LOG_NDELAY, LOG_LOCAL0);
+	strcpy(syslogname, "teapop");
 	tzset();
 
-	while ((ch = getopt(argc, argv, "dDhil:nNp:st:vu")) != -1)
+	while ((ch = getopt(argc, argv, "dDe:hil:nNp:sSt:vu")) != -1)
 		switch (ch) {
 		case 'd':
 			pinfo.autodelete = 1;
@@ -99,6 +101,9 @@ char *argv[];
 		case 'D':
 			pinfo.autodelete = 2;
 			break;
+		case 'e':
+			pinfo.expire = (int)pop_string_dtot(optarg);
+			break;
 		case 'i':
 			pinfo.ignoreimap = 1;
 			break;
@@ -120,6 +125,10 @@ char *argv[];
 		case 's':
 			standalone = 1;
 			break;
+		case 'S':
+			pinfo.ssl = 1;
+			strcpy(syslogname, "teapop_ssl");
+			break;
 		case 't':
 			pinfo.timeout = (int) atoi(optarg);
 			break;
@@ -135,6 +144,7 @@ char *argv[];
 			/* NOT REACHED */
 		}
 
+	openlog(syslogname, LOG_PID | LOG_NDELAY, LOG_LOCAL0);
 	pop_read_pwdinfo(&pinfo);
 
 	pinfo.insck = fileno(stdin);
@@ -147,7 +157,7 @@ char *argv[];
 
 	closelog();
 
-	return retval;
+	return (retval);
 }
 
 int
@@ -160,11 +170,15 @@ POP_INFO *pinfo;
 	signal(SIGALRM, pop_signal_sigalrm);
 
 	/* Make sure stdin is a socket */
-	if ((pop_socket_init(pinfo)) != 0) {
-		fprintf(stderr, "stdin is not a socket\r\n");
-		fprintf(stderr, "If you want to run teapop in standalone " \
-		    "mode you have to use the -s argument.\r\n");
-		return (1);
+	if ((retval = pop_socket_init(pinfo)) != 0) {
+		if (retval == 2) {
+			fprintf(stderr,"connection refused by tcp_wrappers\r\n");
+		} else {
+			fprintf(stderr, "stdin is not a socket\r\n");
+			fprintf(stderr, "If you want to run teapop in standalone " \
+			    "mode you have to use the -s argument.\r\n");
+		}
+		return (retval);
 	}
 
 	switch (pop_auth(pinfo)) {
@@ -200,7 +214,7 @@ POP_INFO *pinfo;
 		break;
 	}
 
-	return 0;
+	return (0);
 }
 
 int
@@ -213,7 +227,7 @@ POP_INFO *pinfo;
 		syslog(LOG_ERR, "Problem when doing fork() : %s",
 		    strerror(errno));
 		perror("fork");
-		return 3;
+		return (3);
 	}
 	setsid();
 
@@ -223,7 +237,7 @@ POP_INFO *pinfo;
 	logpid();
 
 	if ((sckfd = pop_socket_bind()) == 0)
-		return 1;
+		return (1);
 
 	signal(SIGCHLD, pop_signal_sigchld);
 	signal(SIGTERM, pop_signal_sigterm);
@@ -260,14 +274,15 @@ POP_INFO *pinfo;
 
 	/* clean up - closelog done in main() after return */
 	close(sckfd);
-	return(0);
+	return (0);
 }
 
 void
 usage(void)
 {
 
-	printf("Syntax: teapop [-dDhinNsuv] [ -l minutes ] [-p hostname] [-t seconds]\n");
+	printf("Syntax: teapop [-dDhinNsSuv] [-e age] [ -l minutes ] "
+	    "[-p hostname] [-t seconds]\n");
 
 	exit(0);
 }
